CREATE PROCEDURE DOMAIN_Conf (
	@sPrintMe		CHAR(1),
	@sOwnTrans		CHAR(1),
	@sRtnMsg		VARCHAR(255) = NULL OUTPUT,

	/* other vars with descriptions */
	@sConfConst			VARCHAR(255) = 'Y',			/* Configuration Variable we are looking for */
	@iDomainName		INT = 0,					/* This is the DomainUnq of the current Domain - if it's 0 then error */
	@sServerCV			INT = -1,					/* This is the ServerUnq the page is being run on - helps determine which conf var to use (alpha/beta/live/etc) */
	@sGroupList			VARCHAR(255) = ''			/* The list of Groups the user is in -- if any. */
)
WITH ENCRYPTION
AS

/* Additional variables used in this procedure */
DECLARE
	@iRtnCode    		INT,			/* Return 0 for success, prefer positive values for failure */
	@sValue				VARCHAR(255)	/* The value to return */


/* Initialize the main variables in use */
SELECT @iRtnCode = 0



/* Area to perform all qualifications that do not depend on being inside the transaction */
/*		Examples are the existence of the qualified row, duplicate possibility, proper format etc. */
/*		Failures go to Proc_Finalize ...  i.e.  IF NOT EXISTS .... GOTO Proc_Finalize */
/* Make sure there is something for the Conf Var */
IF @sConfConst = '' BEGIN
	SELECT @iRtnCode = -1, @sRtnMsg = '<br>Critical Error: Invalid sConfConst.<br>'
	GOTO Proc_Finalize
END

/* Make sure there is a DomainUnq */
If @iDomainName <= 0 BEGIN
	SELECT @iRtnCode = -2, @sRtnMsg = '<br>Critical Error: Invalid iDomainName.<br>'
	GOTO Proc_Finalize
END



If @sServerCV > -1 
	BEGIN
		IF @sGroupList = '' 
			BEGIN
				SELECT @sValue = Value FROM Configurations (NOLOCK) WHERE
					(ServerUnq = 0 OR ServerUnq = @sServerCV) AND 
					DomainUnq = @iDomainName AND 
					ConfConst = @sConfConst
			END
		ELSE
			BEGIN
				SELECT @sValue = Value FROM Configurations (NOLOCK) WHERE
					(ServerUnq = 0 OR ServerUnq = @sServerCV) AND 
					DomainUnq = @iDomainName AND 
					ConfConst = @sConfConst AND
					GroupUnq IN (@sGroupList)
			END
		SELECT @iRtnCode = 0, @sRtnMsg = @sServerCV
	END
ELSE
	BEGIN
		SELECT @sValue = Value FROM Configurations (NOLOCK) WHERE
			(ServerUnq = 0 OR ServerUnq = 1) AND 
			DomainUnq = @iDomainName AND 
			ConfConst = @sConfConst
		SELECT @iRtnCode = -3, @sRtnMsg = '<br>Critical Error: Invalid sServerCV.<br>Please make sure you have set the correct IP addresses for your servers.<br>Using server #1 as failsafe.<br>'
	END



Proc_Finalize:


/* Create a recordset output if requested */
IF @sPrintMe = 'Y'
	SELECT @sValue "sValues", @iRtnCode "RtnCode", @sRtnMsg "RtnMsg"

RETURN @iRtnCode